@@ -1,7 +1,7 @@
 #
 #	affine.R
 #
-#	$Revision: 1.43 $	$Date: 2013/02/25 07:49:05 $
+#	$Revision: 1.45 $	$Date: 2014/01/31 10:56:14 $
 #
 
 affinexy <- function(X, mat=diag(c(1,1)), vec=c(0,0), invert=FALSE) {
@@ -37,8 +37,7 @@ affinexypolygon <- function(p, mat=diag(c(1,1)), vec=c(0,0),
 "affine.owin" <- function(X,  mat=diag(c(1,1)), vec=c(0,0), ...,
                           rescue=TRUE) {
   verifyclass(X, "owin")
-  if(!is.vector(vec) || length(vec) != 2)
-    stop(paste(sQuote("vec"), "should be a vector of length 2"))
+  vec <- as2vector(vec)
   if(!is.matrix(mat) || any(dim(mat) != c(2,2)))
     stop(paste(sQuote("mat"), "should be a 2 x 2 matrix"))
   # Inspect the determinant
@@ -95,6 +94,7 @@ affinexypolygon <- function(p, mat=diag(c(1,1)), vec=c(0,0),
 
 "affine.ppp" <- function(X, mat=diag(c(1,1)), vec=c(0,0), ...) {
   verifyclass(X, "ppp")
+  vec <- as2vector(vec)
   r <- affinexy(X, mat, vec)
   w <- affine.owin(X$window, mat, vec, ...)
   return(ppp(r$x, r$y, window=w, marks=marks(X, dfok=TRUE), check=FALSE))
@@ -102,8 +102,7 @@ affinexypolygon <- function(p, mat=diag(c(1,1)), vec=c(0,0),
 
 "affine.im" <- function(X,  mat=diag(c(1,1)), vec=c(0,0), ...) {
   verifyclass(X, "im")
-  if(!is.vector(vec) || length(vec) != 2)
-    stop(paste(sQuote("vec"), "should be a vector of length 2"))
+  vec <- as2vector(vec)
   if(!is.matrix(mat) || any(dim(mat) != c(2,2)))
     stop(paste(sQuote("mat"), "should be a 2 x 2 matrix"))
   # Inspect the determinant
@@ -147,7 +146,7 @@ affinexypolygon <- function(p, mat=diag(c(1,1)), vec=c(0,0),
     unitname(W) <- newunits
     # raster for transformed image
     naval <- switch(X$type,
-                    factor={ factor(NA, levels=levels(X)) },
+                    factor= , 
                     integer = NA_integer_,
                     logical = as.logical(NA_integer_),
                     real = NA_real_,
@@ -160,7 +159,14 @@ affinexypolygon <- function(p, mat=diag(c(1,1)), vec=c(0,0),
     yy <- as.vector(rastery.im(Y))
     pre <- affinexy(list(x=xx, y=yy), mat, vec, invert=TRUE)
     # sample original image
-    Y$v[] <- lookup.im(X, pre$x, pre$y, naok=TRUE)
+    if(X$type != "factor") {
+      Y$v[] <- lookup.im(X, pre$x, pre$y, naok=TRUE)
+    } else {
+      lab <- levels(X)
+      lev <- seq_along(lab)
+      Y$v[] <- lookup.im(eval.im(as.integer(X)), pre$x, pre$y, naok=TRUE)
+      Y <- eval.im(factor(Y, levels=lev, labels=lab))
+    }
   }
   return(Y)
 }
@@ -198,6 +204,10 @@ reflect.im <- function(X) {
 }
 
 shiftxy <- function(X, vec=c(0,0)) {
+  if(is.null(vec)) {
+    warning("Null displacement vector; treated as zero")
+    return(X)
+  }
   list(x = X$x + vec[1],
        y = X$y + vec[2])
 }
@@ -226,6 +236,7 @@ shiftxypolygon <- function(p, vec=c(0,0)) {
     } else stop("origin must be a character string or a numeric vector")
     return(shift(X, -locn))
   }
+  vec <- as2vector(vec)
   # Shift the bounding box
   X$xrange <- X$xrange + vec[1]
   X$yrange <- X$yrange + vec[2]
@@ -269,6 +280,7 @@ shiftxypolygon <- function(p, vec=c(0,0)) {
     } else stop("origin must be a character string or a numeric vector")
     vec <- -locn
   }
+  vec <- as2vector(vec)
   # perform shift
   r <- shiftxy(X, vec)
   w <- shift.owin(X$window, vec)
@@ -278,6 +290,19 @@ shiftxypolygon <- function(p, vec=c(0,0)) {
   return(Y)
 }
 
+getlastshift <- function(X) {
+  v <- attr(X, "lastshift")
+  if(is.null(v))
+    stop(paste("Internal error: shifted object of class",
+               sQuote(as.character(class(X))[1]),
+               "does not have \"lastshift\" attribute"),
+         call.=FALSE)
+  if(!(is.numeric(v) && length(v) == 2))
+    stop("Internal error: \"lastshift\" attribute is not a vector",
+         call.=FALSE)
+  return(v)
+}
+
 ### ---------------------- scalar dilation ---------------------------------
 
 scalardilate <- function(X, f, ...) {
@@ -299,7 +324,7 @@ scalardilate.im <- scalardilate.owin <- scalardilate.psp <- scalardilate.ppp <-
   stopifnot(is.finite(f) && f > 0)
   if(!is.null(origin)) {
     X <- shift(X, origin=origin)
-    negorig <- attr(X, "lastshift")
+    negorig <- getlastshift(X)
   } else negorig <- c(0,0)
   Y <- affine(X, mat=diag(c(f, f)), vec = -negorig)
   return(Y)
