@@ -4,7 +4,7 @@
 #
 #    class "fv" of function value objects
 #
-#    $Revision: 1.95 $   $Date: 2013/07/05 06:13:24 $
+#    $Revision: 1.106 $   $Date: 2014/02/16 02:36:57 $
 #
 #
 #    An "fv" object represents one or more related functions
@@ -82,7 +82,7 @@ fv <- function(x, argu="r", ylab=NULL, valu, fmla=NULL,
     if(any(nbg)) desc[nbg] <- ""
   }
   if(!is.null(fname))
-    stopifnot(is.character(fname) && length(fname) == 1)
+    stopifnot(is.character(fname) && length(fname) %in% 1:2)
   # pack attributes
   attr(x, "argu") <- argu
   attr(x, "valu") <- valu
@@ -172,10 +172,7 @@ print.fv <- function(x, ...) {
   if(!is.null(ylab))
     desc <- sprintf(desc, ylab)
   # Labels ..
-  labl <- a$labl
-  # .. may require insertion of function name if it is known
-  if(!is.null(fname <- attr(x, "fname")))
-    labl <- sprintf(labl, fname)
+  labl <- fvlabels(x, expand=TRUE)
   # Start printing
   cat("Entries:\n")
   lablen <- nchar(labl)
@@ -308,11 +305,18 @@ fvnames <- function(X, a=".") {
 fvlabels <- function(x, expand=FALSE) {
   lab <- attr(x, "labl")
   names(lab) <- names(x)
-  if(expand) {
-    # expand plot labels
-    if(!is.null(fname <- attr(x, "fname")))
-      lab <- sprintf(lab, fname)
+  if(expand && !is.null(fname <- attr(x, "fname"))) {
+    ## expand plot labels using function name
+    nstrings <- max(substringcount("%s", lab))
+    ## pad with blanks
+    nextra <- nstrings - length(fname)
+    if(nextra > 0) 
+      fname <- c(fname, rep("", nextra))
+    ## render
+    lab <- do.call(sprintf, append(list(lab), as.list(fname)))
   }
+  ## remove empty space
+  lab <- gsub(" ", "", lab)
   return(lab)
 }
 
@@ -324,6 +328,36 @@ fvlabels <- function(x, expand=FALSE) {
   return(x)
 }
 
+flatfname <- function(x) {
+  fn <- if(is.character(x)) x else attr(x, "fname")
+  if(length(fn) > 1)
+    fn <- paste0(fn[1], "[", paste(fn[-1], collapse=" "), "]")
+  as.name(fn)
+}
+
+makefvlabel <- function(op=NULL, accent=NULL, fname, sub=NULL, argname="r") {
+  ## de facto standardised label
+  a <- "%s"
+  if(!is.null(accent)) 
+    a <- paste0(accent, paren(a))     ## eg hat(%s)
+  if(!is.null(op))
+    a <- paste0("bold", paren(op), "~", a)  ## eg bold(var)~hat(%s)
+  if(is.null(sub)) {
+    if(length(fname) != 1) {
+      a <- paste0(a, "[%s]")
+      a <- paren(a, "{")
+    }
+  } else {
+    if(length(fname) == 1) {
+      a <- paste0(a, paren(sub, "["))
+    } else {
+      a <- paste0(a, paren("%s", "["), "^", paren(sub, "{"))
+      a <- paren(a, "{")
+    }
+  } 
+  a <- paste0(a, paren(argname))
+  return(a)
+}
 
 fvlabelmap <- local({
   magic <- function(x) {
@@ -381,7 +415,7 @@ bind.fv <- function(x, y, labl=NULL, desc=NULL, preferred=NULL) {
   if(is.fv(y)) {
     # y is already an fv object
     ay <- attributes(y)
-    if(ax$fname != ay$fname) {
+    if(!identical(ax$fname, ay$fname)) {
       # x and y represent different functions
       # expand the labels separately 
       fvlabels(x) <- fvlabels(x, expand=TRUE)
@@ -443,7 +477,8 @@ bind.fv <- function(x, y, labl=NULL, desc=NULL, preferred=NULL) {
           c(ax$labl, labl),
           c(ax$desc, desc),
           unitname=unitname(x),
-          fname=ax$fname)
+          fname=ax$fname,
+          yexp=ax$yexp)
   return(z)
 }
 
@@ -598,6 +633,47 @@ rebadge.fv <- function(x, new.ylab, new.fname,
   return(x)
 }
 
+## common invocations to label a function like Kdot or Kcross
+rebadge.as.crossfun <- function(x, main, sub=NULL, i, j) {
+  i <- make.parseable(i)
+  j <- make.parseable(j)
+  if(is.null(sub)) {
+    ylab <- substitute(main[i, j](r),
+                       list(main=main, i=i, j=j))
+    fname <- c(main, paste0("list", paren(paste(i, j, sep=","))))
+    yexp <- substitute(main[list(i, j)](r),
+                       list(main=main, i=i, j=j))
+  } else {
+    ylab <- substitute(main[sub, i, j](r),
+                       list(main=main, sub=sub, i=i, j=j))
+    fname <- c(main, paste0("list", paren(paste(sub, i, j, sep=","))))
+    yexp <- substitute(main[list(sub, i, j)](r),
+                       list(main=main, sub=sub, i=i, j=j))
+  }
+  y <- rebadge.fv(x, new.ylab=ylab, new.fname=fname, new.yexp=yexp)
+  return(y)
+}
+
+rebadge.as.dotfun <- function(x, main, sub=NULL, i) {
+  i <- make.parseable(i)
+  if(is.null(sub)) {
+    ylab <- substitute(main[i ~ dot](r),
+                       list(main=main, i=i))
+    fname <- c(main, paste0(i, "~symbol(\"\\267\")"))
+    yexp <- substitute(main[i ~ symbol("\267")](r),
+                       list(main=main, i=i))
+  } else {
+    ylab <- substitute(main[sub, i ~ dot](r),
+                       list(main=main, sub=sub, i=i))
+    fname <- c(main, paste0("list",
+                            paren(paste0(sub, ",",
+                                         i, "~symbol(\"\\267\")"))))
+    yexp <- substitute(main[list(sub, i ~ symbol("\267"))](r),
+                       list(main=main, sub=sub, i=i))
+  }
+  y <- rebadge.fv(x, new.ylab=ylab, new.fname=fname, new.yexp=yexp)
+  return(y)
+}
 
 # subset extraction operator
 "[.fv" <-
@@ -648,7 +724,8 @@ rebadge.fv <- function(x, new.ylab, new.fname,
                labl=attr(x, "labl")[selected],
                desc=attr(x, "desc")[selected],
                unitname=attr(x, "units"),
-               fname=attr(x,"fname"))
+               fname=attr(x,"fname"),
+               yexp=attr(x, "yexp"))
   # carry over preferred names, if possible
   dotn <- fvnames(x, ".")
   fvnames(result, ".") <- dotn[dotn %in% colnames(result)]
@@ -829,7 +906,7 @@ prefixfv <- function(x, tagprefix="", descprefix="", lablprefix=tagprefix,
   oldtags <- names(x)[relevant]
   newtags <- paste(tagprefix, oldtags, sep="")
   newlabl <- paste(lablprefix, att$labl[relevant], sep="")
-  newdesc <- paste(descprefix, att$desc[relevant], sep="")
+  newdesc <- paste(descprefix, att$desc[relevant])
   y <- rebadge.fv(x, tags=oldtags,
                   new.desc=newdesc,
                   new.labl=newlabl,
@@ -875,14 +952,60 @@ reconcile.fv <- function(...) {
   return(z)
 }
 
-as.function.fv <- function(x, ..., value, extrapolate=FALSE) {
+as.function.fv <- function(x, ..., value=".y", extrapolate=FALSE) {
+  trap.extra.arguments(...)
+  # extract function argument
   xx <- with(x, .x)
-  yy <- if(!missing(value) && value %in% names(x)) x[[value]] else with(x, .y)
-  endrule <- if(!extrapolate) 1 else 2
-  f <- approxfun(xx, yy, rule=endrule)
+  # extract all function values 
+  yy <- as.data.frame(x)[, fvnames(x, "*"), drop=FALSE]
+  # determine which value(s) to supply
+  if(!is.character(value))
+    stop("value should be a character string or vector specifying columns of x")
+  if(!all(value %in% colnames(yy))) {
+    expandvalue <- try(fvnames(x, value))
+    if(!inherits(expandvalue, "try-error")) {
+      value <- expandvalue
+    } else stop("Unable to determine columns of x")
+  }
+  yy <- yy[,value]
+  argname <- fvnames(x, ".x")
+  endrule <- if(extrapolate) 1 else 2
+  if(length(value) == 1) {
+    # make a single 'approxfun' and return it
+    f <- approxfun(xx, yy, rule=endrule)
+    # magic
+    names(formals(f))[1] <- argname
+    body(f)[[4]] <- as.name(argname)
+  } else {
+    # make a list of 'approxfuns' 
+    funs <- lapply(yy,
+                   function(z, u, endrule) { approxfun(x=u, y=z, rule=endrule)},
+                   u=xx,
+                   endrule=endrule)
+    # return a function which selects the appropriate 'approxfun' and executes
+    f <- function(x, what=value) {
+      what <- match.arg(what)
+      funs[[what]](x)
+    }
+    # magic
+    formals(f)[[2]] <- value
+    names(formals(f))[1] <- argname
+    body(f)[[3]][[2]] <- as.name(argname)
+  }
+  class(f) <- c("fvfun", class(f))
+  attr(f, "fname") <- attr(x, "fname")
+  attr(f, "yexp") <- attr(x, "yexp")
   return(f)
 }
 
+print.fvfun <- function(x, ...) {
+  y <- args(x)
+  yexp <- as.expression(attr(x, "yexp"))
+  body(y) <- as.name(paste("Returns interpolated value of", yexp))
+  print(y, ...)
+  return(invisible(NULL))
+}
+
 findcbind <- function(root, depth=0, maxdepth=1000) {
   # recursive search through a parse tree to find calls to 'cbind'
   if(depth > maxdepth) stop("Reached maximum depth")
@@ -1043,3 +1166,4 @@ conform.ratfv <- function(x) {
   attr(x, "denominator") <- den
   return(x)
 }
+
