@@ -2,7 +2,7 @@
 #
 #    hardcore.S
 #
-#    $Revision: 1.7 $	$Date: 2012/06/28 04:20:25 $
+#    $Revision: 1.9 $	$Date: 2014/01/31 08:04:21 $
 #
 #    The Hard core process
 #
@@ -28,10 +28,37 @@ Hardcore <- local({
          },
          par    = list(hc = NULL),  # filled in later
          parnames = "hard core distance", 
+         selfstart = function(X, self) {
+           # self starter for Hardcore
+           nX <- npoints(X)
+           if(nX < 2) {
+             # not enough points to make any decisions
+             return(self)
+           }
+           md <- min(nndist(X))
+           if(md == 0) {
+             warning(paste("Pattern contains duplicated points:",
+                           "impossible under Hardcore model"))
+             return(self)
+           }
+           if(!is.na(hc <- self$par$hc)) {
+             # value fixed by user or previous invocation
+             # check it
+             if(md < hc)
+               warning(paste("Hard core distance is too large;",
+                             "some data points will have zero probability"))
+             return(self)
+           }
+           # take hc = minimum interpoint distance * n/(n+1)
+           hcX <- md * nX/(nX+1)
+           Hardcore(hc = hcX)
+       },
          init   = function(self) {
            hc <- self$par$hc
-           if(!is.numeric(hc) || length(hc) != 1 || hc <= 0)
-             stop("hard core distance hc must be a positive number")
+           if(length(hc) != 1)
+             stop("hard core distance must be a single value")
+           if(!is.na(hc) && !(is.numeric(hc) && hc > 0))
+             stop("hard core distance hc must be a positive number, or NA")
          },
          update = NULL,       # default OK
          print = NULL,        # default OK
@@ -71,11 +98,19 @@ Hardcore <- local({
          # second Mayer cluster integral
          hc <- self$par$hc
          return(pi * hc^2)
+       },
+       Percy=function(d, coeffs, par, ...) {
+         ## term used in Percus-Yevick type approximation
+         H <- par$hc
+         t <- abs(d/(2*H))
+         t <- pmin.int(t, 1)
+         y <- 2 * H^2 * (pi - acos(t) + t * sqrt(1 - t^2))
+         return(y)
        }
   )
   class(BlankHardcore) <- "interact"
   
-  Hardcore <- function(hc) {
+  Hardcore <- function(hc=NA) {
     instantiate.interact(BlankHardcore, list(hc=hc))
   }
 
