@@ -4,7 +4,7 @@
 #	A class 'ppp' to define point patterns
 #	observed in arbitrary windows in two dimensions.
 #
-#	$Revision: 4.84 $	$Date: 2013/01/08 07:16:16 $
+#	$Revision: 4.89 $	$Date: 2014/02/07 08:58:53 $
 #
 #	A point pattern contains the following entries:	
 #
@@ -24,7 +24,7 @@
 #			'marks' attached to the corresponding points.	
 #	
 #--------------------------------------------------------------------------
-ppp <- function(x, y, ..., window, marks, check=TRUE ) {
+ppp <- function(x, y, ..., window, marks, check=TRUE, drop=TRUE) {
   # Constructs an object of class 'ppp'
   #
   if(!missing(window))
@@ -78,14 +78,15 @@ ppp <- function(x, y, ..., window, marks, check=TRUE ) {
   } else nout <- 0
   # initialise ppp object
   pp <- list(window=window, n=n, x=x, y=y)
-  # coerce marks to appropriate forma
+  # coerce marks to appropriate format
   if(missing(marks))
     marks <- NULL
   if(is.hyperframe(marks)) 
     stop("Hyperframes of marks are not implemented for ppp objects; use ppx")
   if(is.matrix(marks)) 
     marks <- as.data.frame(marks)
-  if(is.data.frame(marks)) {
+  ## drop dimensions?
+  if(drop && is.data.frame(marks)) {
     nc <- ncol(marks)
     if(nc == 0)
       marks <- NULL
@@ -316,9 +317,12 @@ cobble.xy <- function(x, y, f=ripras, fatal=TRUE, ...) {
                      check=FALSE)
           } else {
             # assume i is a subset index
-            if(x$n == 0)
+            nx <- x$n
+            if(nx == 0)
               return(x)
-            subset <- i
+            subset <- seq_len(nx)[i]
+            if(any(is.na(subset)))
+              stop("Index out of bounds in [.ppp", call.=FALSE)
             x <- ppp(x$x[subset], x$y[subset], window=x$window,
                      marks=marksubset(x$marks, subset),
                      check=FALSE)
@@ -382,45 +386,69 @@ scanpp <- function(filename, window, header=TRUE, dir="", multitype=FALSE) {
 
 #-------------------------------------------------------------------
 
-print.ppp <- function(x, ...) {
-  verifyclass(x, "ppp")
-  ism <- is.marked(x, dfok=TRUE)
-  cat(paste(if(ism) "marked" else NULL,
-            "planar point pattern:",
-            x$n,
-            ngettext(x$n, "point", "points"),
-            "\n"))
-  if(ism) {
-    mks <- marks(x, dfok=TRUE)
-    if(is.data.frame(mks)) {
-      # data frame of marks
-      cat(paste("Mark variables: ", paste(names(mks), collapse=", "), "\n"))
+print.ppp <- local({
+
+  graceful <- function(prefix, strings) {
+    shortblurb <- paste(prefix, paste(strings, collapse=", "), "\n")
+    if(nchar(shortblurb) < options("width")[[1]]) {
+      cat(shortblurb)
     } else {
-      # vector of marks
-      if(is.factor(mks)) {
-        cat("multitype, with ")
-        cat(paste("levels =", paste(levels(mks), collapse="\t"),"\n"))
+      cat(paste(prefix,"\n"))
+      print(strings, quote=FALSE)
+    }
+    return(invisible(NULL))
+  }
+    
+  print.ppp <- function(x, ...) {
+    verifyclass(x, "ppp")
+    ism <- is.marked(x, dfok=TRUE)
+    cat(paste(if(ism) "marked" else NULL,
+              "planar point pattern:",
+              x$n,
+              ngettext(x$n, "point", "points"),
+              "\n"))
+    if(ism) {
+      mks <- marks(x, dfok=TRUE)
+      if(is.data.frame(mks)) {
+        ## data frame of marks
+        graceful("Mark variables:", names(mks))
       } else {
-        cat(paste("marks are",
-                  if(is.numeric(mks)) "numeric,",
-                  "of type", sQuote(typeof(mks)), "\n"))
+        ## vector of marks
+        if(is.factor(mks)) {
+          graceful("Multitype, with levels =", levels(mks))
+        } else {
+          ## Numeric, or could be dates
+          if(inherits(mks, "Date")) {
+            desc <- paste("dates, of class", sQuote("Date"))
+          } else if(inherits(mks, "POSIXt")) {
+            desc <- paste("dates, of class", sQuote("POSIXt"))
+          } else {
+            desc <- paste0(if(is.numeric(mks)) "numeric, ",
+                           "of storage type ", sQuote(typeof(mks)))
+            styp <- typeof(mks)
+          }
+          cat(paste("marks are", desc, "\n"))
+        }
       }
     }
+    print(x$window)
+    if(!is.null(rejects <- attr(x, "rejects"))) {
+      nrejects <- rejects$n
+      cat(paste("\n***",
+                nrejects,
+                ngettext(nrejects, "illegal point", "illegal points"),
+                "stored in",
+                paste("attr(,", dQuote("rejects"), ")", sep=""),
+                "***\n"))
+    }
+    if(!is.null(info <- attr(x, "info")) && inherits(info, "rmhInfoList"))
+      cat("\nPattern was generated by Metropolis-Hastings simulation.\n")
+    return(invisible(NULL))
   }
-  print(x$window)
-  if(!is.null(rejects <- attr(x, "rejects"))) {
-    nrejects <- rejects$n
-    cat(paste("\n***",
-              nrejects,
-              ngettext(nrejects, "illegal point", "illegal points"),
-              "stored in",
-              paste("attr(,", dQuote("rejects"), ")", sep=""),
-              "***\n"))
-  }
-  if(!is.null(info <- attr(x, "info")) && inherits(info, "rmhInfoList"))
-    cat("\nPattern was generated by Metropolis-Hastings simulation.\n")
-  return(invisible(NULL))
-}
+
+  print.ppp
+})
+
 
 summary.ppp <- function(object, ..., checkdup=TRUE) {
   verifyclass(object, "ppp")
