@@ -1,7 +1,7 @@
 #
 # split.ppp.R
 #
-# $Revision: 1.18 $ $Date: 2011/08/08 06:21:48 $
+# $Revision: 1.20 $ $Date: 2014/01/13 00:32:05 $
 #
 # split.ppp and "split<-.ppp"
 #
@@ -264,9 +264,12 @@ print.summary.splitppp <- function(x, ...) {
     fsplit <- f
   } else if(is.factor(f)) {
     lev <- levels(f)
-    fsplit <- factor(rep(lev, unlist(lapply(x, npoints))), levels=lev)
+    fsplit <- factor(rep.int(lev, unlist(lapply(x, npoints))), levels=lev)
   }
   attr(x, "fsplit") <- fsplit
   x
 }
   
+density.splitppp <- function(x, ...) {
+  as.listof(lapply(x, density, ...))
+}
