@@ -207,11 +207,11 @@ plot(Y, main="Choi-Hall data sharpening", pch=".")
   W <- grow.rectangle(as.rectangle(letterR), 1)
   X <- superimpose(runifpoint(300, letterR),
                    runifpoint(50, W), W=W)
-  plot(W, main="clusterset(X)")
-  plot(clusterset(X, fast=TRUE), add=TRUE, chars=c("o", "+"), cols=1:2)
+  plot(W, main="clusterset(X, 'm')")
+  plot(clusterset(X, 'marks', fast=TRUE), add=TRUE, chars=c("o", "+"), cols=1:2)
   plot(letterR, add=TRUE)
   plot(W, main="clusterset(X, 'd')")
-  plot(clusterset(X, "d", exact=FALSE), add=TRUE)
+  plot(clusterset(X, 'domain', exact=FALSE), add=TRUE)
   plot(letterR, add=TRUE)
   par(owpa)
 
