@@ -34,6 +34,8 @@ local({
   if(any(nw5 != nw5P))
     stop("nnwhich.ppp(k=5) does not agree with pairdist")
 
+  # Three dimensions
+
   X <- runifpoint3(42)
 
   nn <- nndist(X)
@@ -118,7 +120,22 @@ local({
   if(any(ncw != cdw))
     stop("For sorted data, nncross()$which does not agree with apply(crossdist(), 1, which.min)")
 
-  # test of nngrid.h and knngrid.h
+  # sanity check for nncross with k > 1
+  ndw <- nncross(X, Y, k=1:4)$which
+  if(any(is.na(ndw)))
+    stop("NA's returned by nncross.ppp(k > 1)$which")
+  ndw <- nncross(X, Y, k=1:4, what="which")
+  if(any(is.na(ndw)))
+    stop("NA's returned by nncross.ppp(k > 1, what='which')")
+  
+  # test of correctness for nncross with k > 1
+  flipcells <- flipxy(cells)
+  calcwhich <- nncross(cells, flipcells, k=1:4, what="which")
+  truewhich <- t(apply(crossdist(cells,flipcells), 1, order))[,1:4]
+  if(any(calcwhich != truewhich))
+    stop("nncross(k > 1) gives wrong answer")
+  
+  # test of agreement between nngrid.h and knngrid.h
   #    dimyx=23 (found by trial-and-error) ensures that there are no ties 
   a <- as.matrix(nnmap(cells, what="which", dimyx=23))
   b <- as.matrix(nnmap(cells, what="which", dimyx=23, k=1:2)[[1]])
@@ -282,6 +299,14 @@ local({
   ss <- summary(ljtrmod)
 })
 
+local({
+  # From Ege
+  # Degenerate but non-null argument 'covariates'
+  xx <- list()
+  names(xx) <- character(0)
+  fit <- ppm(cells, ~x, covariates = xx)
+  st <- summary(fit) 
+})
 #
 #   tests/ppmgam.R
 #
@@ -298,6 +323,31 @@ local({
   v <- vcov(fit)
 })
 
+#
+#  ppmlogi.R
+#
+# Tests of ppm(method='logi')
+#
+# $Revision: 1.2 $  Date$
+#
+
+require(spatstat)
+local({
+  fit <- ppm(cells, ~x, method="logi")
+  f <- fitted(fit)
+  p <- predict(fit)
+  fitS <- ppm(cells, ~x, Strauss(0.08), method="logi")
+  fS <- fitted(fitS)
+  pS <- predict(fitS)
+  if(spatstat.options("allow.logi.influence")) {
+    a <- leverage(fit)
+    b <- influence(fit)
+    d <- dfbetas(fit)
+    aS <- leverage(fitS)
+    bS <- influence(fitS)
+    dS <- dfbetas(fitS)
+  }
+})
 # ppmmarkorder.R
 # $Revision: 1.2 $  $Date: 2011/12/05 07:29:16 $
 # Test that predict.ppm, plot.ppm and plot.fitin
@@ -352,6 +402,23 @@ local({
 
 })
 
+#
+# tests/prediction.R
+#
+# Things that might go wrong with predict()
+#
+#  $Revision: 1.1 $ $Date: 2013/11/12 16:06:11 $
+#
+
+require(spatstat)
+
+local({
+  # test of 'covfunargs'
+  f <- function(x,y,a){ y - a }
+  fit <- ppm(cells, ~x + f, covariates=list(f=f), covfunargs=list(a=1/2))
+  p <- predict(fit)
+})
+
 #
 #  tests/rmhAux.R
 #
@@ -1153,10 +1220,12 @@ local({
   plot(alltypes(amacrine, "J"))
   plot(alltypes(amacrine, pcfcross))
 })
-# test of case where mark levels contain illegal characters
+## tests involving strange mark values
+## $Revision: 1.2 $ $Date: 2014/01/08 11:15:05 $
 
 require(spatstat)
 local({
+  ## ppm() where mark levels contain illegal characters
   hyphenated <- c("a", "not-a")
   spaced <- c("U", "non U")
   suffixed <- c("a+", "a*")
@@ -1180,6 +1249,18 @@ local({
   tryit(spaced, amacrine, irad, hrad)
   tryit(suffixed, amacrine, irad, hrad)
   tryit(charred, amacrine, irad, hrad)
+
+  ## marks which are dates
+  X <- cells
+  n <- npoints(X)
+  endoftime <- rep(ISOdate(2001,1,1), n)
+  eotDate   <- rep(as.Date("2001-01-01"), n)
+  markformat(endoftime)
+  markformat(eotDate)
+  marks(X) <- endoftime
+  print(X)
+  Y <- X %mark% data.frame(id=1:42, date=endoftime, dd=eotDate)
+  print(Y)
 })
 #
 # test cases where there are no (rows or columns of) marks
@@ -1334,9 +1415,28 @@ local({
 })
 
 
+## badwindowcheck.R
+## $Revision: 1.2 $  $Date: 2014/01/27 07:18:41 $
+##
+
 require(spatstat)
-source("badwindow.R")
-owinpolycheck(W,verbose=FALSE)
+local({
+  ## Simple example of self-crossing polygon
+  x <- read.table("selfcross.txt", header=TRUE)
+  ## Auto-repair
+  w <- owin(poly=x)
+
+  ## Real data involving various quirks
+  b <- read.table("badwindow.txt", header=TRUE)
+  b <- split(b, factor(b$i))
+  b <- lapply(b, function(z) { as.list(z[,-3]) })
+  ## make owin without checking
+  W <- owin(poly=b, check=FALSE)
+  ## Apply stringent checks
+  owinpolycheck(W,verbose=FALSE)
+  ## Auto-repair
+  W2 <- owin(poly=b)
+})
 
 
 
@@ -1450,7 +1550,7 @@ local({
 #
 #  Thanks to Marcelino de la Cruz
 #
-#  $Revision: 1.8 $  $Date: 2013/10/06 08:44:28 $
+#  $Revision: 1.9 $  $Date: 2014/02/05 09:30:38 $
 #
 
 require(spatstat)
@@ -1492,6 +1592,11 @@ tessr4 <- tess(tiles=list(r00, r05,r50,r55))
 puntosr4 <- split(puntos, tessr4, drop=TRUE)
 split(puntos, tessr4, drop=TRUE) <- puntosr4
 
+## More headaches with mark format
+A <- runifpoint(10)
+B <- runifpoint(10)
+AB <- split(superimpose(A=A, B=B))
+
 })
 #
 #  tests/imageops.R
@@ -1591,7 +1696,7 @@ local({
 #
 #  Thanks to Ege Rubak
 #
-#  $Revision: 1.4 $  $Date: 2013/09/20 09:01:34 $
+#  $Revision: 1.5 $  $Date: 2014/01/24 05:59:31 $
 #
 
 require(spatstat)
@@ -1648,9 +1753,18 @@ local({
   if(disagree(v, vc))
     stop("Disagreement between vcov.ppm algorithms 'vector' and 'vectorclip' for Geyer model")
 
-  # test of pairwise.family$delta2
+
+  ## tests of 'deltasuffstat' code
+  ##     Handling of offset terms
+  modelH <- ppm(cells, ~x, Hardcore(0.05))
+  a <- vcov(modelH, generic=TRUE) ## may fall over
+  b <- vcov(modelH, generic=FALSE)
+  if(disagree(a, b))
+    stop("Disagreement between vcov.ppm algorithms for Hardcore model")
+  
+  ##     Correctness of pairwise.family$delta2
   modelZ <- ppm(amacrine, ~1, MultiStrauss(radii=matrix(0.1, 2, 2)))
-  b <- vcov(modelZ)
+  b <- vcov(modelZ, generic=FALSE)
   g <- vcov(modelZ, generic=TRUE)
   if(disagree(b, g))
     stop("Disagreement between vcov.ppm algorithms for MultiStrauss model")
@@ -1798,3 +1912,60 @@ local({
 
 
 
+#
+# tests/mppm.R
+#
+# Basic tests of mppm
+#
+# $Revision: 1.1 $ $Date: 2013/11/10 08:59:08 $
+# 
+
+require(spatstat)
+
+local({
+# test interaction formulae and subfits
+fit1 <- mppm(Points ~ group, simba, hyperframe(po=Poisson(), str=Strauss(0.1)),
+            iformula=~ifelse(group=="control", po, str))
+fit2 <- mppm(Points ~ group, simba, hyperframe(po=Poisson(), str=Strauss(0.1)),
+            iformula=~id * str)
+fit3 <- mppm(Points ~ group, simba, hyperframe(po=Poisson(), pie=PairPiece(c(0.05,0.1))), iformula=~I((group=="control") * po) + I((group=="treatment") * pie))
+fit1
+fit2
+fit3
+subfits(fit1)
+subfits(fit2)
+subfits(fit3)
+
+# test handling of offsets and zero cif values in mppm
+
+ data(waterstriders)
+ H <- hyperframe(Y = waterstriders)
+ mppm(Y ~ 1,  data=H, Hardcore(1.5))
+ mppm(Y ~ 1,  data=H, StraussHard(7, 1.5))
+})
+# tests/ppx.R
+#
+# Test operations for ppx objects
+#
+#  $Revision: 1.1 $ $Date: 2013/11/19 03:36:27 $
+#
+
+require(spatstat)
+
+local({
+  df <- data.frame(x=c(1,2,2,1), y=c(1,2,3,1), z=c(2,3,4,2))
+  X <- ppx(data=df, coord.type=rep("s", 3), domain=box3())
+  unique(X)
+  duplicated(X)
+  multiplicity(X)
+})
+## tests/percy.R
+## Tests of Percus-Yevick approximations
+##    $Revision: 1.1 $ $Date: 2014/01/31 11:49:46 $
+
+require(spatstat)
+local({
+  fit <- ppm(swedishpines, ~1, DiggleGatesStibbard(6))
+  K <- Kmodel(fit)
+})
+
